#!/usr/bin/env node
/*jslint node:true,indent:4*/

"use strict";

var ArgParser = require("argparse").ArgumentParser,
    async = require("async"),
    pack = require("../pack"),
    minify = require("../minify-batch"),
    buildDoc = require("../builddoc"),
    path = require("path"),
    fs = require("fs"),
    parser;

parser = new ArgParser({
    version: "0.1",
    addHelp: true,
    description: "ePages 6 node.js helper"
});

parser.addArgument(
    [ "-j", "--pack-js" ],
    {
        nargs: 0,
        required: false,
        help: "Pack the JavaScript files"
    }
);

parser.addArgument(
    [ "-c", "--pack-css" ],
    {
        nargs: 0,
        required: false,
        help: "Pack the LESS and CSS files"
    }
);

parser.addArgument(
    [ "-m", "--minify" ],
    {
        nargs: 0,
        required: false,
        help: "Minify all files"
    }
);

parser.addArgument(
    [ "--minify-filter" ],
    {
        nargs: 1,
        required: false,
        help: "Filter files to minify"
    }
);

parser.addArgument(
    [ "-t", "--store-type" ],
    {
        nargs: 1,
        required: false,
        help: "The store type"
    }
);

parser.addArgument(
    [ "-s", "--store-version" ],
    {
        nargs: 1,
        required: false,
        help: "The version"
    }
);

parser.addArgument(
    [ "-p", "--profile" ],
    {
        nargs: 1,
        required: false,
        help: "The profile"
    }
);

parser.addArgument(
    [ "-e", "--file-extension" ],
    {
        nargs: 1,
        required: false,
        help: "The file extension used for the css package file"
    }
);

parser.addArgument(
    [ "-u", "--uid" ],
    {
        nargs: 1,
        required: false,
        help: "The user id of the owner of the created files and directories."
    }
);

parser.addArgument(
    [ "-g", "--gid" ],
    {
        nargs: 1,
        required: false,
        help: "The group id of the group of the created files and directories."
    }
);

parser.addArgument(
    [ "-w", "--webroot" ],
    {
        nargs: 1,
        required: false,
        help: "The webroot"
    }
);

parser.addArgument(
    [ "-d", "--build-doc" ],
    {
        nargs: 0,
        required: false,
        help: "Build the documentation."
    }
);

parser.addArgument(
    [ "-i", "--include" ],
    {
        nargs: 1,
        required: false,
        help: "A JSON file which enumerates the files to include in the documentation."
    }
);

parser.addArgument(
    [ "-o", "--output" ],
    {
        nargs: 1,
        required: false,
        help: "The output folder for the documentation"
    }
);

parser.addArgument(
    [ "-f", "--cartridge-folder" ],
    {
        nargs: 1,
        required: false,
        help: "The folder containing the cartridges, typically /srv/epages/eproot/Cartridges"
    }
);
var buffer = "";

var args = parser.parseArgs();

var storeType = args.store_type ? args.store_type[0] : undefined;
var profile = args.profile ? args.profile[0] : undefined;
var version = args.store_version ? args.store_version[0] : undefined;
var webroot = args.webroot ? args.webroot[0] : undefined;

var fileExtension = args.file_extension ? args.file_extension[0] : "css";

var include = args.include ? args.include[0] : undefined;
var output = args.output ? args.output[0] : undefined;
var cartridgeFolder = args.cartridge_folder ? args.cartridge_folder[0] : undefined;

var uid = args.uid ? parseInt(args.uid[0], 10) : undefined;
var gid = args.gid ? parseInt(args.gid[0], 10) : undefined;
var minify_filter = args.minify_filter ? args.minify_filter[0] : undefined;

var startTime = new Date().getTime();
async.parallel([
    function (callback) {
        async.series([
            function (callback) {
                async.parallel([
                    function (callback) {
                        if (args.pack_js) {
                            if (!profile) {
                                console.error("[ERROR] Packing JS");
                                callback("Please provide profile");
                                return;
                            }

                            if (!version) {
                                console.error("[ERROR] Packing JS");
                                callback("Please provide version");
                                return;
                            }

                            if (!webroot) {
                                console.error("[ERROR] Packing JS");
                                callback("Please provide webroot");
                                return;
                            }

                            if (!storeType) {
                                console.error("[ERROR] Packing JS");
                                callback("Please provide storeType");
                                return;
                            }

                            pack.packJs({
                                "storeType": storeType,
                                "profile": profile,
                                "version": version,
                                "webroot": webroot,
                                "uid": uid,
                                "gid": gid
                            }, function (error) {
                                if (error) {
                                    console.error("[ERROR] Packing JS");
                                    buffer = " === Packing JS === \n";
                                    callback(error);
                                } else {
                                    callback();
                                }
                            });
                        } else {
                            callback();
                        }
                    },

                    function (callback) {
                        if (args.pack_css) {
                            if (!profile) {
                                console.error("[ERROR] Packing CSS");
                                callback("Please provide profile");
                                return;
                            }

                            if (!version) {
                                console.error("[ERROR] Packing CSS");
                                callback("Please provide version");
                                return;
                            }

                            if (!webroot) {
                                console.error("[ERROR] Packing CSS");
                                callback("Please provide webroot");
                                return;
                            }

                            if (!storeType) {
                                console.error("[ERROR] Packing CSS");
                                callback("Please provide storeType");
                                return;
                            }

                            pack.packLess({
                                "storeType": storeType,
                                "profile": profile,
                                "version": version,
                                "webroot": webroot,
                                "fileExtension": fileExtension,
                                "uid": uid,
                                "gid": gid
                            }, function (error) {
                                if (error) {
                                    console.error("[ERROR] Packing CSS");
                                    buffer = " === Packing CSS === \n";
                                    callback(error);
                                } else {
                                    callback();
                                }
                            });
                        } else {
                            callback();
                        }
                    }
                ], callback);
            },

            function (callback) {
                if (args.minify) {
                    if (!version) {
                        console.error("[ERROR] Minify");
                        callback("Please provide version");
                        return;
                    }

                    if (!webroot) {
                        console.error("[ERROR] Minify");
                        callback("Please provide webroot");
                        return;
                    }

                    if (!storeType) {
                        console.error("[ERROR] Minify");
                        callback("Please provide storeType");
                        return;
                    }

                    try {
                        var target = path.join(webroot, "StoreTypes", version, storeType);
                        minify(
                            path.join(target, "lib"),
                            path.join(target, "lib-min"),
                            { "uid": uid, "gid": gid, "filter": minify_filter},
                            callback
                        );
                    } catch (e) {
                        console.error("[ERROR] Minify");
                        buffer = " === Minifying === \n";
                        callback(e);
                    }
                } else {
                    callback();
                }
            }
        ], callback);
    },

    function (callback) {
        if (args.build_doc) {
            if (!include) {
                console.log("[ERROR] Build documentation");
                callback("Please provide include.json");
                return;
            }

            output = output || path.join(process.env.EPAGES_WEBROOT, "Docs", "JavaScript");
            cartridgeFolder = cartridgeFolder || process.env.EPAGES_CARTRIDGES;

            buildDoc({
                "input": include,
                "output": output,
                "cartridges": cartridgeFolder,
                "uid": uid,
                "gid": gid
            }, function (error) {
                if (error) {
                    console.log("[ERROR] Build documentation");
                }

                callback(error);
            });
        } else {
            callback();
        }
    }
], function (error) {
    /*jslint regexp:true*/
    // To let me replace the UTC format e.g. ".157Z"
    var stopTime = new Date().getTime(),
        logFile = path.join(webroot, "/../Log/Epnode6.log"),
        logTime = new Date().toISOString().replace(/T/, ' ').replace(/\..+/, ''),
        logMessage;

    console.log("Need " + ((stopTime - startTime) / 1000) + "s");

    if (error) {
        console.error("[ERROR] " + error);
        logMessage = logTime + " " + buffer + error + "\n\n";
        /*jslint stupid: true*/
        // To let me write the error log
        fs.appendFileSync(logFile, logMessage);

        process.exit(1);
    } else {
        process.exit();
    }
});