/*jslint indent:4,node:true,stupid:true*/
(function () {

    'use strict';

    var path = require('path'),
        grunt = require('grunt'),
        fs = require('fs'),
        buildTemplateModule,
        compileTemplateFiles,
        fixPath;

    buildTemplateModule = function (module, source) {
        var html,
            wrapper;

        html = fs.readFileSync(source).toString();

        // This is the JS code used to wrap a template. `##MODULE_NAME##` is replaced by the
        // template name and `##HTML##` is replaced by the actual content of the template.
        wrapper = 'define("$tmpl!##MODULE_NAME##", ["jquery/tmpl"], function ($) {'
                + '    $.template("##TEMPLATE_NAME_ONE##", "##HTML##");'
                + '    var tmpl = $.template("##TEMPLATE_NAME_TWO##", "##HTML##");'
                + '    return function (data, options) {'
                + '        return $.tmpl(tmpl, data, options);'
                + '    };'
                + '});';

        // The double quotes and line breaks must be escaped, everything else can be left
        // unchanged.
        html = html.replace(/"/g, '\\"')
                   .replace(/[\n\r]\s*/g, '');

        return wrapper
            .replace('##MODULE_NAME##', module)
            .replace('##TEMPLATE_NAME_ONE##', module.replace(/\//g, '.'))
            .replace('##TEMPLATE_NAME_TWO##', module)
            .replace(/##HTML##/g, html);
    };

    compileTemplateFiles = function (base, source) {
        var dir = path.dirname(source),
            file = path.basename(source, '.js'),
            concatenatedContent = '',
            includedFiles = [];

        base = base.replace(/\\/g, '/');
        if (base.charAt(base.length - 1) !== '/') {
            base += '/';
        }

        grunt.file.expand(path.join(dir, file + '*.tmpl.html'))
            .forEach(function (templateFile) {
                var module = templateFile.replace(base, "").replace(".tmpl.html", ""),
                    templateContent,
                    compiledTemplateFile;

                // Fix paths on Windows.
                module = fixPath(path.relative(base, templateFile).replace(/\.tmpl\.html$/, ""));

                // The actual content of the template file is decorated with some
                // boilerplate code by this function.
                templateContent = buildTemplateModule(module, templateFile);

                compiledTemplateFile = path.join(dir, path.basename(templateFile, ".html") + ".js");

                includedFiles.push(compiledTemplateFile);

                concatenatedContent += templateContent;
                fs.writeFileSync(compiledTemplateFile, templateContent);
            });

        includedFiles.push(source);
        concatenatedContent += fs.readFileSync(source);

        return {
            includedFiles: includedFiles,
            concatenatedContent: concatenatedContent
        };
    };

    fixPath = function (p) {
        return p.replace(new RegExp(path.sep === '\\' ? '\\\\' : path.sep, 'g'), '/');
    };

    module.exports = {
        compileTemplateFiles: compileTemplateFiles,
        fixPath: fixPath
    };

}());
